/* pscal - a PostScript calendar generator */

/* Christopher Phillips */
/* pefv700@utpe.pe.utexas.edu */

/*
 * Copyright (C) 1992, Christopher Phillips
 * Permission to use and modify this software and its 
 * documentation for any purpose other than its incorporation 
 * into a commercial product is hereby granted without fee. 
 * Permission to copy and distribute this software and its 
 * documentation only for non-commercial use is also granted 
 * without fee, provided, however, that the above copyright 
 * notice appear in all copies, that both that copyright 
 * notice and this permission notice appear in supporting 
 * documentation. The author makes no representations about 
 * the suitability of this software for any purpose. It is 
 * provided ``as is'' without express or implied warranty. 
 */

/*
 * This is pscal, my PostScript calendar generator.
 * It formats either the current month, a specified month,
 * or every month of a specified year.  Three types of holidays
 * are understood:
 	* those with fixed dates -- Christmas
	* those with fixed days	 -- Thanksgiving
	* all others require a function to determine their date -- Easter
 * Note that everything is U.S.-biased including the change from
 * the Julian to the Gregorian calendar.
 * Some support for fixed-date Jewish holidays is included.
 *
 * Basically I wanted to reproduce a ``standard'' (to me) calendar.
 * Improvements/additions are welcomed.
 */

/* Modified by Marty Olevitch (marty@cosray2.wustl.edu) to allow reading
 * customized holiday and event information from a file. Dec 1998 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "defs.h"
#include "structs.h"
#include "months.h"
#include "prolog.h"

extern void	do_page(int, int, int);
extern void	do_trailer(int);
extern void	get_easter_date(int);
extern int	first_from_easter(int);
extern void	printerr(char *);
extern void	usage(void);

int read_dates(char *evtsfile);
char *Prolog = PROLOG;

char	*progname;	/* name of this program */
time_t	now;		/* the current time */

int	main(int argc, char **argv)
{
    	int		c;
	extern int	optind;
	extern char	*optarg;

	char 		*evtsfile = NULL;
	int		num_month;	/* the month we want */
	int		first_day = -1;	/* the day the first falls on */
	int		year;		/* the year we want */
	int		i;
	char		*bad_arg;	/* check for bad month and year */
	struct tm	*tp;		/* the current time */

	int		do_year = 0;
	int		do_month = 0;

	progname = argv[0];

	if (argc > 9) {
	    usage();
	}

	while ((c = getopt(argc, argv, "f:m:p:y:")) != EOF) {
	    switch (c) {
		case 'f':
		    evtsfile = optarg;
		    if (read_dates(evtsfile)) {
			printerr(evtsfile);
		    }
		    break;
		case 'm':
		    do_month = 1;
		    num_month = (int)strtol(optarg, &bad_arg, 10) - 1;
		    if (*bad_arg || num_month<JANUARY || num_month>DECEMBER) {
			usage();
		    }
		    break;
		case 'p':
		    Prolog = optarg;
		    break;
		case 'y':
		    do_year = 1;
		    year = (int)strtol(optarg, &bad_arg, 10);
		    if (*bad_arg || year < 1) {
			usage();
		    }
		    break;
		default:
		    usage();
		    break;
	    }
	}

	if (do_month && !do_year) {
	    usage();
	}

	if (!do_month && !do_year) {
	    /* Format only this month. */

	    if ((now = time(NULL)) == -1) {
		    printerr("can't get current time");
	    }
	    tp = localtime(&now);
	    year = tp->tm_year + 1900;

	    /* Is this a leap year? */
	    if (isleap(year))
		    month[FEBRUARY].length++;
	    if (tp->tm_mon == MARCH || tp->tm_mon == APRIL)
		    get_easter_date(year);

	    if ((first_day = tp->tm_wday - tp->tm_mday % DAYS_PER_WEEK + 1)
	      < SUNDAY)
		    first_day += DAYS_PER_WEEK;
	    do_page(tp->tm_mon, year, first_day);
	} else {
	    /* Is this a leap year? */
	    if (isleap(year))
		    month[FEBRUARY].length++;
	    /*
	     * More ugliness required if
	     * LAST_MONTH_OF_JULIAN != FIRST_MONTH_OF_GREGORIAN
	     */
	    if (year == LAST_YEAR_OF_JULIAN)
		    month[LAST_MONTH_OF_JULIAN].length
		      -= FIRST_DAY_OF_GREGORIAN - LAST_DAY_OF_JULIAN - 1;
		    
	    get_easter_date(year);

	    if (!do_month) {
		/* do whole year */
		num_month = JANUARY;
	    }

	    /*
	     * Find what day the first was using the date of Easter Sunday
	     */
	    first_day = first_from_easter(num_month);

	    if (!do_month) {
		/* Do each month for the specified year. */
		do_page(JANUARY, year, first_day);
		for (i = FEBRUARY; i <= DECEMBER; i++) {
		    first_day = NEXT_FIRST(i - 1, first_day);
		    do_page(i, year, first_day);
		}
	    } else {
		/* do specified month and year */
		do_page(num_month, year, first_day);
	    }
	}

	do_trailer(do_month ? 1 : 12);

	exit(0);
}
