#ifndef PSCAL_DEFS_H
#define PSCAL_DEFS_H

#define SUNDAY		0
#define MONDAY		1
#define TUESDAY		2
#define WEDNESDAY	3
#define THURSDAY	4
#define FRIDAY		5
#define SATURDAY	6
#define LAST		5

#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif

#define ARRSIZ(a)	(sizeof (a) / sizeof (a)[0])
#define MAX(a, b)	((a) > (b) ? (a) : (b))

#define DAYS_PER_WEEK	7

/*
 * Julian and Gregorian section
 */

#define JANUARY		0
#define FEBRUARY	1
#define MARCH		2
#define APRIL		3
#define MAY		4
#define JUNE		5
#define JULY		6
#define AUGUST		7
#define SEPTEMBER	8
#define OCTOBER		9
#define NOVEMBER	10
#define DECEMBER	11

#define LAST_MONTH_OF_JULIAN		SEPTEMBER
#define LAST_DAY_OF_JULIAN		2
#define LAST_YEAR_OF_JULIAN		1752
#define FIRST_MONTH_OF_GREGORIAN	SEPTEMBER
#define FIRST_DAY_OF_GREGORIAN		14
#define FIRST_YEAR_OF_GREGORIAN		1752

/*
 * The following two macros take month ``m'' which has its first day
 * on day ``f'' and give back the day the first falls on for the
 * next and previous months.
 */
#define NEXT_FIRST(m, f)	(((f) + month[m].length) % DAYS_PER_WEEK)
#define PREV_FIRST(m, f)	\
	(((f) - (month[ (m) == JANUARY ? DECEMBER : (m - 1)].length \
	  % DAYS_PER_WEEK) + DAYS_PER_WEEK) % DAYS_PER_WEEK)

/*
 * How many weeks will we need on our calendar for month ``m''?
 */
#define WEEKS(m, f)	((month[m].length + (f) + DAYS_PER_WEEK - 1) \
			  / DAYS_PER_WEEK)

#define isleap(y)	((y) % 4 == 0 \
			  && ((y) <= 1752 || (y) % 100 || (y) % 400 == 0))

/*
 * Jewish section
 */

#define NISAN	0
#define IYAR	1
#define SIVAN	2
#define TAMMUZ	3
#define AB	4
#define ELLUL	5
#define TISHRI	6
#define HESHVAN	7
#define KISLEV	8
#define TEBETH	9
#define SHEBAT	10
#define ADAR	11
#define ADAR_II	12

#define TIME_0_DAY		MONDAY
#define TIME_0_PARTS		5604
#define PARTS_PER_HOUR		1080
#define PARTS_PER_DAY		(1080 * 24)
#define DAYS_PER_MONTH		29
#define EXTRA_PARTS_PER_MONTH	13753
#define MONTHS_PER_CYCLE	235
#define YEARS_PER_CYCLE		19
#define	isjleap(y)		(((7 * (y) + 1) % YEARS_PER_CYCLE) < 7)

#ifndef STREQ
#define STREQ(a, b)	(strcmp((a), (b)) == 0)
#endif

#ifndef CHOP
/* CHOP - replace last character of a string */
#define CHOP(s, c)	((s)[strlen((s))-1] = (c))
#endif

#ifndef NCHOP
/* NCHOP - only replace the last char if it is a '\n' */
#define NCHOP(s, c)	if ((s)[strlen((s))-1] == '\n')	{	\
				(s)[strlen((s))-1] = (c);	\
			}
#endif

#endif /* PSCAL_DEFS_H */
