#include "defs.h"
#include "structs.h"

extern struct month month[];

/* struct used to hold the date of Easter Sunday */
struct {
	int	month;
	int	day;
} date_of_easter;

/*
 * Since we know Easter is a Sunday, we can find out the
 * day on which the first of any month falls.
 */
int	first_from_easter(int m)
{
	int	day = 0;
	int	i;

	if (m <= date_of_easter.month) {
		for (i = m; i < date_of_easter.month; i++)
			day += month[i].length % DAYS_PER_WEEK;
		day += date_of_easter.day - 1;
		day = DAYS_PER_WEEK - day % DAYS_PER_WEEK;
	} else {
		for (i = date_of_easter.month + 1; i < m; i++)
			day += month[i].length % DAYS_PER_WEEK;
		day += month[date_of_easter.month].length
		  - date_of_easter.day + 1;
	}
	return day % DAYS_PER_WEEK;
}

/* determine the date of Easter */
/* Source: The Art of Computer Programming, Donald Knuth, V. 1, 155-156 */
void	get_easter_date(int yr)
{
	int	golden;
	int	century;
	int	epact;
	int	Gregorian_correction;
	int	Clavian_correction;
	int	day;
	int	extra_days;

	golden = yr % 19 + 1;
	if (yr > FIRST_YEAR_OF_GREGORIAN) {
		century = yr / 100 + 1;
		Gregorian_correction = 3 * century / 4 - 12;
		Clavian_correction = (8 * century + 5) / 25 - 5;
		extra_days = 5 * yr / 4 - Gregorian_correction - 10;
		epact = (11 * golden + 20 + Clavian_correction
		  - Gregorian_correction) % 30;
		if (epact < 0)
			epact += 30;
		if (epact == 25 && golden > 11 || epact == 24)
			epact++;
	} else {
		extra_days = 5 * yr / 4;
		epact = (11 * golden - 4) % 30 + 1;
	}
	if ((day = 44 - epact) < 21)
		day += 30;
	if ((day += 7 - (extra_days + day) % 7) > 31) {
		date_of_easter.month = APRIL;
		date_of_easter.day = day - 31;
	} else {
		date_of_easter.month = MARCH;
		date_of_easter.day = day;
	}
}

/* determine if Easter Sunday occurs this month */
int	easter(int mon)
{
	return (date_of_easter.month == mon) ? date_of_easter.day : 0;
}

/* determine when Palm Sunday occurs */
int	palm(int mon)
{
	int	prev_sunday;	/* the Sunday before Easter Sunday */

	prev_sunday = date_of_easter.day - (DAYS_PER_WEEK - SUNDAY);
	if (date_of_easter.month == mon && prev_sunday > 0)
		return prev_sunday;
	else if (mon == MARCH && date_of_easter.month == APRIL
	  && prev_sunday <= 0)
		return month[MARCH].length + prev_sunday;
	else
		return 0;
}

/* determine when Good Friday occurs */
int	good(int mon)
{
	int	prev_friday;	/* the Friday before Easter Sunday */

	prev_friday = date_of_easter.day - (DAYS_PER_WEEK - FRIDAY);
	if (date_of_easter.month == mon && prev_friday > 0)
		return prev_friday;
	else if (mon == MARCH && date_of_easter.month == APRIL
	  && prev_friday <= 0)
		return month[MARCH].length + prev_friday;
	else
		return 0;
}
