#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <pwd.h>
#ifdef unix
#include <unistd.h>
#endif
#include "defs.h"
#include "structs.h"
#include "holidays.h"

#include <linklist.h>

extern struct date	Getdate(int, int, int);
extern void		printerr(char *);

extern struct month	month[];
extern time_t		now;

extern char *Prolog;

unsigned long day96_to_sec96(int day);
int read_dates(char *evtsfile);
int str_to_month(char *s);

/* linked list IDs */
static int Evt_id;	/* one-time events */
static int Hol_id;	/* holidays (annual evts) */
static int Mon_id;	/* monthly stuff */

#define ERR_NEXT_TOK(p)	(p) = strtok(NULL, "\t");			\
			if ((p) == NULL) {				\
			    fprintf(stderr, "line: %d\n", linecnt);	\
			    return -1;					\
			}

#define NEXT_TOK(p)	(p) = strtok(NULL, "\t")

#define ERR_MALLOC(p, n)	(p) = malloc((n));			\
				if ((p) == NULL) {			\
				    fprintf(stderr, "line: %d\n", linecnt);\
				    return -1;				\
				}
				     	
#define MALLOC(p, n)	(p) = malloc((n))

static char *Blurbtext = NULL;

#ifdef NOTDEF
/*
 * Check if a fixed holiday has already been formatted for a given day -
 * this is used to avoid redundant text if an "observed" holiday falls on
 * its fixed date.
 */
int	already_fixed(int mon, int day, char *name1)
{
	int			i;
	struct fixed_holiday	*fp;

	for (i = 0, fp = fixed_holiday; i < ARRSIZ(fixed_holiday); i++, fp++)
		if (fp->month == mon && fp->day == day
		  && strncmp(fp->name1, name1, 5) == 0)
			return TRUE;

	return FALSE;
}
#endif

void
do_doy(int mon, int yr, int first)
{
    int doy;
    int day96;
    int i;
    int maxday;
    u_long sec96;

    doy = 1;
    for (i=JANUARY; i < mon; i++) {
	doy += month[i].length;
    }

    (void)printf("SetHolidayFont\n");
    maxday = month[mon].length;
    for (i=1; i <= maxday; i++) {
	(void)printf("[(doy %d)] %d %d Holiday\n", doy, first, i);
	day96 = day_of_year_96(yr, doy);
	(void)printf("[(day96 %d)] %d %d Holiday\n", day96, first, i);
	sec96 = day96_to_sec96(day96);
	(void)printf("[(sec96 %lu)] %d %d Holiday\n", sec96, first, i);
	doy++;
    }
}

void
do_one_time_events(int mon, int yr, int first)
{
    int i;
    struct one_time_event *ep;

    (void)printf("SetEventFont\n");
    (void)ll_first(Evt_id);
    do {
	ep = ll_val(Evt_id);
	if (ep == NULL) {
	    continue;
	}
	if (ep->year == yr && ep->month == mon) {
	    (void)printf("[");
	    if (ep->name3) {
		(void)printf("(%s) ", ep->name3);
	    }
	    if (ep->name2) {
		(void)printf("(%s) ", ep->name2);
	    }
	    (void)printf("(%s)] %d %d Holiday\n", ep->name1, first, ep->day);
	}
    } while (ll_next(Evt_id) == 1);
}

void
do_monthly_stuff(int mon, int yr)
{
    char dc, ic;
    int i;
    struct monthly_stuff *mp;

    (void)ll_first(Mon_id);
    do {
	mp = ll_val(Mon_id);
	if (mp == NULL) {
	    continue;
	}
	if (mp->year == yr && mp->month == mon) {
	    dc = mp->discrim_changed ? '*' : '-';
	    ic = mp->iigain_changed ? '*' : '-';
	    (void)printf("(Discrim %d %c %d) (IIgain %d %c %d) MonthlyText\n",
	    	mp->discrim[0], dc, mp->discrim[1], 
		mp->iigain[0], ic, mp->iigain[1]);
	}
    } while (ll_next(Mon_id) == 1);
}

/* format text for all holidays */
void	do_holidays(int mon, int yr, int first)
{
	int			i;
	int			day;
	struct fixed_holiday	*fp;
	struct moving_holiday	*mp;
	struct strange_holiday	*sp;
	struct date		date;

	(void)printf("SetHolidayFont\n");
	/* look for a fixed holiday this month */
	(void)ll_first(Hol_id);
	do {
	    fp = ll_val(Hol_id);
	    if (fp == NULL) {
		continue;
	    }
	    if (fp->month == mon) {
		(void)printf("[");
		if (fp->name2)
			(void)printf("(%s) ", fp->name2);
		/*
		 * Note that the switch from Julian to Gregorian
		 * may mess things up.
		 */
		(void)printf("(%s)] %d %d Holiday\n", fp->name1,
		  first, fp->day);
	    }
	} while (ll_next(Hol_id) == 1);

#ifdef NOTDEF
	/* look for a moving holiday this month */
	for (i = 0, mp = moving_holiday; i < ARRSIZ(moving_holiday); i++, mp++)
		if (mp->month == mon) {
			day = (mp->which - (mp->day >= first)) * DAYS_PER_WEEK
			  + mp->day - first + 1;
			/*
			 * Don't format the "observed" text when the holiday
			 * cooincides with the "observed" date.
			 */
			if (already_fixed(mon, day, mp->name1))
				continue;
			(void)printf("[");
			if (mp->name3)
				(void)printf("(%s) ", mp->name3);
			if (mp->name2)
				(void)printf("(%s) ", mp->name2);
			if (mp->which == LAST)
				while (day > month[mon].length)
					day -= DAYS_PER_WEEK;
			(void)printf("(%s)] %d %d Holiday\n", mp->name1, first,
			  day);
		}

	/* look for a strange holiday this month */
	for (i = 0, sp = strange_holiday; i < ARRSIZ(strange_holiday); i++,sp++)
		if (sp->month == mon && (day = (*sp->func)(mon)) > 0)
			(void)printf("[(%s)] %d %d Holiday\n", sp->name, first,
			  day);

	/* Jewish holidays */
	/* The algorithms only seem to work to the year 300 */
	if (yr <= 300)
		return;

	/* look for a fixed holiday this month */
	for (i = 0, fp = Jewish_fixed; i < ARRSIZ(Jewish_fixed); i++, fp++) {
		date = Getdate(fp->month, fp->day, yr);
		if (date.year != yr) {
			date = Getdate(fp->month, fp->day, yr - 1);
			if (date.year != yr)
				continue;
		}
		if (date.year == yr && date.month == mon) {
			/*
			 * Note that the switch from Julian to Gregorian
			 * may mess things up.
			 */
			if (yr == LAST_YEAR_OF_JULIAN
			  && mon == LAST_MONTH_OF_JULIAN
			  && date.day > LAST_DAY_OF_JULIAN)
				date.day -= FIRST_DAY_OF_GREGORIAN
				  - LAST_DAY_OF_JULIAN - 1;
			(void)printf("[");
			if (fp->name2)
				(void)printf("(%s) ", fp->name2);
			(void)printf("(%s)] %d %d Holiday\n", fp->name1,
			  first, date.day);
		}
	}
#endif /* NOTDEF */
}

/* format the page title, e.g., (January 1992) */
void	do_title(int mon, int year)
{
	char	s[6];

	(void)sprintf(s, "%d", year);
	(void)printf("SetTitleFont\n");
	(void)printf("(%s %s) Title\n", month[mon].name, s);
	if (Blurbtext != NULL) {
	    struct tm *t;
	    time_t tt;
	    tt = time(NULL);
	    t = localtime(&tt);
	    (void)printf("SetBlurbFont\n");
	    (void)printf(
	      "(Copyright (C) %d LEXAS, Washington U in St Louis) (%s) Blurb\n",
	    	1900 + t->tm_year, Blurbtext);
	}
	do_monthly_stuff(mon, year);
}

/* format the grid of lines for the page */
void	do_lines(void)
{
	(void)printf("gsave\n");
	(void)printf("	Border\n");
	(void)printf("	HorizontalLines\n");
	(void)printf("	VerticalLines\n");
	(void)printf("grestore\n");
}

/* format the column headings, currently the days of the week */
void	do_headings(void)
{
	(void)printf("SetHeadingFont\n");
	(void)printf("Headings\n");
}

/* the prologue */
void	do_prologue(void)
{
	FILE		*fp;
	char		buf[BUFSIZ];

	if ((fp = fopen(Prolog, "r")) == NULL) {
	    	printerr(Prolog);
	}
	while (fgets((void *)buf, sizeof buf, fp)) {
		if (fputs((void *)buf, stdout) == EOF) {
			break;
		}
	}
	if (ferror(fp))
		printerr("fgets/fputs error");
}

void	numbers(int maxday, int begin, int first)
{
	(void)printf("SetNumberFont\n");
	(void)printf("gsave\n");
	(void)printf("	%d %d %d LeftMargin BoxWidth add %s\n",
	  maxday, begin, first, "NumberRightMargin sub");
	(void)printf("	TopLine HeadingHeight sub NumberBaseline sub\n");
	(void)printf("	BoxWidth BoxHeight RightMargin\n");
	(void)printf("	Numbers\n");
	(void)printf("grestore\n");
}

/* format the date numbers */
void	do_numbers(int mon, int yr, int first)
{
	if (yr == LAST_YEAR_OF_JULIAN && mon == LAST_MONTH_OF_JULIAN) {
		numbers(LAST_DAY_OF_JULIAN, 1, first);
		numbers(month[FIRST_MONTH_OF_GREGORIAN].length
		  + FIRST_DAY_OF_GREGORIAN - LAST_DAY_OF_JULIAN - 1,
		  FIRST_DAY_OF_GREGORIAN, (first + 2) % DAYS_PER_WEEK);
	} else
		numbers(month[mon].length, 1, first);
}

/* trailer */
void	do_trailer(int pages)
{
	(void)printf("%%%%Trailer\n");
	(void)printf("%%%%Pages: %d\n", pages);
}

/* format inset months */
void	do_small_months(int mon, int yr, int first)
{
	int	prev_mon;
	int	next_mon;
	int	prev_first;
	int	next_first;
	int	on_top;

	/*
	 * If the two top-left boxes are empty, the inset months go there,
	 * else they go in the two bottom-right boxes.
	 */
	on_top = first > MONDAY;
	if ((prev_mon = mon - 1) < JANUARY)
		prev_mon = DECEMBER;
	if ((next_mon = mon + 1) > DECEMBER)
		next_mon = JANUARY;
	prev_first = PREV_FIRST(mon, first);
	next_first = NEXT_FIRST(mon, first);

	if (yr == LAST_YEAR_OF_JULIAN && prev_mon == LAST_MONTH_OF_JULIAN)
		(void)printf("2 1 %d (%s) ", prev_first, month[prev_mon].name);
	else
		(void)printf("%d 1 %d (%s) ", month[prev_mon].length,
		  prev_first, month[prev_mon].name);
	if (on_top)
		(void)printf("LeftMargin TopLine HeadingHeight sub\n");
	else
		(void)printf("LeftMargin BoxWidth %d mul add BottomMargin %s\n",
		  DAYS_PER_WEEK - 2, "BoxHeight add");
	(void)printf("SmallMonth\n");
	if (yr == LAST_YEAR_OF_JULIAN && prev_mon == LAST_MONTH_OF_JULIAN) {
		(void)printf("30 14 %d ", (prev_first + 2) % DAYS_PER_WEEK);
		(void)printf("LeftMargin BoxWidth %d mul add BottomMargin %s\n",
		  DAYS_PER_WEEK - 2, "BoxHeight add");
		(void)printf("SmallNumbers\n");
	}
	if (yr == LAST_YEAR_OF_JULIAN && next_mon == LAST_MONTH_OF_JULIAN)
		(void)printf("2 1 %d (%s) ", next_first, month[next_mon].name);
	else
		(void)printf("%d 1 %d (%s) ", month[next_mon].length,
		  next_first, month[next_mon].name);
	if (on_top)
		(void)printf("LeftMargin BoxWidth add TopLine %s\n",
		  "HeadingHeight sub");
	else
		(void)printf("LeftMargin BoxWidth %d mul add BottomMargin %s ",
		  DAYS_PER_WEEK - 1, "BoxHeight add");
	(void)printf("SmallMonth\n");
	if (yr == LAST_YEAR_OF_JULIAN && prev_mon == LAST_MONTH_OF_JULIAN) {
		(void)printf("30 14 %d ", (next_first + 2) % DAYS_PER_WEEK);
		(void)printf("LeftMargin BoxWidth add TopLine %s\n",
		  "HeadingHeight sub");
		(void)printf("SmallNumbers\n");
	}
}

/* format filling of otherwise-empty boxes */
void	do_fills(int mon, int yr, int first)
{
	int	on_top;
	int	last;
	int	day1;
	int	days;

	on_top = first > MONDAY;
	if ((last = NEXT_FIRST(mon, first) - 1) < SUNDAY)
		last += DAYS_PER_WEEK;

	/* boxes on the top row */
	if (days = first - (on_top ? TUESDAY : SUNDAY)) {
		day1 = 1 - days;
		(void)printf("%d %d %d Fill\n", days, day1, first);
	}

	/* boxes on the bottom row */
	if (days = (on_top ? SATURDAY : THURSDAY) - last) {
		day1 = month[mon].length + 1;
		if (yr == LAST_YEAR_OF_JULIAN && mon == LAST_MONTH_OF_JULIAN) {
			(void)printf("%d 20 %d Fill\n", DAYS_PER_WEEK, first);
			(void)printf("%d 27 %d Fill\n", DAYS_PER_WEEK, first);
		} else
			(void)printf("%d %d %d Fill\n", days, day1, first);
	}
}

/* format page for a given month and year */
void	do_page(int mon, int yr, int first)
{
	static int	prologue_included = 0;
	static int	page_number = 0;
	int		weeks;
	struct passwd	*pw;

	if (!prologue_included) {
		(void)printf("%%!PS-Adobe-1.0\n");
		(void)printf("%%%%Title: (none)\n");
		(void)printf("%%%%Creator: ");
#ifdef unix
		if (pw = getpwuid(getuid()))
			(void)printf("%s (%s)\n", pw->pw_name, pw->pw_gecos);
		else
#endif
			(void)printf("(none)\n");
		(void)printf("%%%%CreationDate: %s",
		  now >= 0 ? ctime(&now) : "(none)");
		(void)printf("%%%%Pages: (atend)\n");
		(void)printf("%%%%EndComments\n");

		/* we only insert the prologue once */
		do_prologue();
		prologue_included = 1;
	}
	page_number++;
	(void)printf("%%%%BeginPage: %d %d\n", page_number, page_number);
	(void)printf("save\n");
	(void)printf("Landscape\n");
	(void)printf("InitialHoliday\n");
	weeks = MAX(5, (month[mon].length + first - 1) / DAYS_PER_WEEK + 1);
	(void)printf("%d WeekSizes\n", weeks);
	do_title(mon, yr);
	/*do_fills(mon, yr, first); */
	do_lines();
	do_headings();
	do_numbers(mon, yr, first);
	/* do_small_months(mon, yr, first); */
	do_doy(mon, yr, first);
	do_one_time_events(mon, yr, first);
	do_holidays(mon, yr, first);
	(void)printf("restore\n");
	(void)printf("showpage\n");
	(void)printf("%%%%EndPage\n");
}

/* sc_day_of_year_96 - given the year and day-of-year, return the number of
 * days from Jan. 1, 1996 (inclusive). The year may be in the form 1998,
 * 1999, 2000, 2001, ... or in the form 98, 99, 0, 1 ... Years less than
 * 1996 or greater than 2050 are rejected and -1 is returned. */
int
day_of_year_96(int yr, int doy)
{
    int d;
    int nleap;	/* no. of leap days since 1996 */
    int nyr;

    if (yr < 50) {
	/* assume it is 00, 01, 02... for years >= 2000 */
	yr += 2000;
    } else if (yr < 100) {
	/* 96, 97, 98 ... */
	yr += 1900;
    } else if (yr < 1996 | yr > 2050) {
	/* don't want to handle these */
	return -1;
    }

    if (yr == 1996) {
	return doy;
    }

    nleap = ((yr - 1997) / 4) + 1;
    nyr = yr - 1996;
    d = (365 * nyr) + nleap + doy;
    return d;
}

/* sc_day96_to_sec96 - given a day-of-year-since-1996 return the number of
 * seconds since 1996. The first day is number 1, but the first second is
 * number 0. When more leap seconds are declared, they must be added
 * after the current ones. */
unsigned long
day96_to_sec96(int day)
{
    unsigned long sec;
 
    sec = (day - 1) * 86400L;
 
    if (day >= 548) {   /* July 1997 leap second */
        sec++;
    }
 
    return sec;
}

/* read_dates - read the file containing the holidays, events, monthly
 * stuff, etc */
int 
read_dates(char *evtsfile)
{
    char *cp;
    FILE *fp;
#define LINELEN 512
    char line[LINELEN];
    int linecnt = 0;
    struct one_time_event *evtp;
    struct monthly_stuff *monp;
    struct fixed_holiday *holp;
    int ret;

    if (evtsfile == NULL) {
	return -1;
    }
    fp = fopen(evtsfile, "r");
    if (fp == NULL) {
	return -1;
    }

    Evt_id = ll_new_list();
    if (Evt_id < 0) {
	fprintf(stderr,"Can't make event linked list\n");
	return -1;
    }
    Hol_id = ll_new_list();
    if (Hol_id < 0) {
	fprintf(stderr,"Can't make holiday linked list\n");
	return -1;
    }
    Mon_id = ll_new_list();
    if (Mon_id < 0) {
	fprintf(stderr,"Can't make monthly linked list\n");
	return -1;
    }

    while (fgets(line, LINELEN, fp) != NULL) {
	linecnt++;
	cp = strtok(line, "\t");
	if (cp == NULL) {
	    return -1;
	}
	if (cp[0] == '#') {
	    /* comment */
	    continue;
	}
	if (STREQ(cp, "blurb")) {
	    /* set the 'blurb' at the top of each calendar page */
	    ERR_NEXT_TOK(cp);
	    ERR_MALLOC(Blurbtext, (strlen(cp) + 1));
	    sprintf(Blurbtext, "%s", cp);
	    NCHOP(Blurbtext, '\0');
	    /* fprintf(stderr, "blurb: %s\n", Blurbtext); */
	} else if (STREQ(cp, "holiday")) {
	    /* holiday - an annual event */
	    ERR_MALLOC(holp, (sizeof(struct fixed_holiday)));
	    ERR_NEXT_TOK(cp);
	    holp->month = str_to_month(cp);
	    if (holp->month < 0) {
		return -1;
	    }
	    ERR_NEXT_TOK(cp);
	    holp->day = atol(cp);
	    ERR_NEXT_TOK(cp);
	    NCHOP(cp, '\0');
	    ERR_MALLOC(holp->name1, (STRLEN));
	    strncpy(holp->name1, cp, STRLEN);
	    NEXT_TOK(cp);
	    if (cp != NULL) {
		NCHOP(cp, '\0');
		ERR_MALLOC(holp->name2, (STRLEN));
		strncpy(holp->name2, cp, STRLEN);
	    } else {
		holp->name2 = NULL;
	    }
	    ret = ll_insert(Hol_id, holp);
	    if (ret) {
		fprintf(stderr, "Bad holiday ll_insert() at line %d\n", 
		    linecnt);
		return -1;
	    }
	} else if (STREQ(cp, "discrim")) {
	    /* monthly discrim and gain values */
	    ERR_MALLOC(monp, (sizeof(struct monthly_stuff)));
	    ERR_NEXT_TOK(cp);
	    monp->year = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    monp->month = str_to_month(cp);
	    if (monp->month < 0) {
		return -1;
	    }
	    ERR_NEXT_TOK(cp);
	    monp->discrim[0] = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    monp->discrim_changed = (cp[0] == '*') ? 1 : 0;
	    ERR_NEXT_TOK(cp);
	    monp->discrim[1] = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    monp->iigain[0] = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    monp->iigain_changed = (cp[0] == '*') ? 1 : 0;
	    ERR_NEXT_TOK(cp);
	    monp->iigain[1] = atoi(cp);
	    ret = ll_insert(Mon_id, monp);
	    if (ret) {
		fprintf(stderr, "Bad monthly ll_insert() at line %d\n", 
		    linecnt);
		return -1;
	    }
	} else if (STREQ(cp, "evt")) {
	    /* one-time event */
	    ERR_MALLOC(evtp, (sizeof(struct one_time_event)));
	    ERR_NEXT_TOK(cp);
	    evtp->year = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    evtp->month = str_to_month(cp);
	    if (evtp->month < 0) {
		return -1;
	    }
	    ERR_NEXT_TOK(cp);
	    evtp->day = atoi(cp);
	    ERR_NEXT_TOK(cp);
	    NCHOP(cp, '\0');
	    ERR_MALLOC(evtp->name1, (STRLEN));
	    strncpy(evtp->name1, cp, STRLEN);
	    NEXT_TOK(cp);
	    if (cp != NULL) {
		NCHOP(cp, '\0');
		ERR_MALLOC(evtp->name2, (STRLEN));
		strncpy(evtp->name2, cp, STRLEN);
		NEXT_TOK(cp);
		if (cp != NULL) {
		    NCHOP(cp, '\0');
		    ERR_MALLOC(evtp->name3, (STRLEN));
		    strncpy(evtp->name3, cp, STRLEN);
		} else {
		    evtp->name3 = NULL;
		}
	    } else {
		evtp->name2 = NULL;
	    }
	    ret = ll_insert(Evt_id, evtp);
	    if (ret) {
		fprintf(stderr, "Bad event ll_insert() at line %d\n", 
		    linecnt);
		return -1;
	    }
	} else {
	    /* unknown keyword */
	    fprintf(stderr, "Unknown keyword (%s) at line %d\n",
	    	cp, linecnt);
	    return -1;
	}
    }

#ifdef NOTDEF
    /* debug code to see if the linked lists got set correctly */
    (void)ll_first(Hol_id);
    do {
	holp = ll_val(Hol_id);
	fprintf(stderr, "%d/%d %s", holp->month, holp->day,
	    holp->name1);
	if (holp->name2) {
	    fprintf(stderr, " %s", holp->name2);
	}
	fprintf(stderr, "\n");
    } while (ll_next(Hol_id) == 1);

    (void)ll_first(Mon_id);
    do {
	monp = ll_val(Mon_id);
	fprintf(stderr, "%d/%d %d%c%d %d%c%d\n",
	    monp->month, monp->year,
	    monp->discrim[0], (monp->discrim_changed) ? '*' : '-',
	    	monp->discrim[1],
	    monp->iigain[0], (monp->iigain_changed) ? '*' : '-',
	    	monp->iigain[1]);
    } while (ll_next(Mon_id) == 1);

    (void)ll_first(Evt_id);
    do {
	evtp = ll_val(Evt_id);
	fprintf(stderr, "%d/%d/%d %s", evtp->month, evtp->day, evtp->year,
	    evtp->name1);
	if (evtp->name2) {
	    fprintf(stderr, " %s", evtp->name2);
	    if (evtp->name3) {
		fprintf(stderr, " %s", evtp->name3);
	    }
	}
	fprintf(stderr, "\n");
    } while (ll_next(Evt_id) == 1);
#endif

    return 0;
}

int
str_to_month(char *s)
{
    if (STREQ(s, "JANUARY") || STREQ(s, "Jan") || STREQ(s, "jan")) {
	return JANUARY;
    } else if (STREQ(s, "FEBRUARY") || STREQ(s, "Feb") || STREQ(s, "feb")) {
	return FEBRUARY;
    } else if (STREQ(s, "MARCH") || STREQ(s, "Mar") || STREQ(s, "mar")) {
	return MARCH;
    } else if (STREQ(s, "APRIL") || STREQ(s, "Apr") || STREQ(s, "apr")) {
	return APRIL;
    } else if (STREQ(s, "MAY") || STREQ(s, "May") || STREQ(s, "may")) {
	return MAY;
    } else if (STREQ(s, "JUNE") || STREQ(s, "Jun") || STREQ(s, "jun")) {
	return JUNE;
    } else if (STREQ(s, "JULY") || STREQ(s, "Jul") || STREQ(s, "jul")) {
	return JULY;
    } else if (STREQ(s, "AUGUST") || STREQ(s, "Aug") || STREQ(s, "aug")) {
	return AUGUST;
    } else if (STREQ(s, "SEPTEMBER") || STREQ(s, "Sep") || STREQ(s, "sep")) {
	return SEPTEMBER;
    } else if (STREQ(s, "OCTOBER") || STREQ(s, "Oct") || STREQ(s, "oct")) {
	return OCTOBER;
    } else if (STREQ(s, "NOVEMBER") || STREQ(s, "Nov") || STREQ(s, "nov")) {
	return NOVEMBER;
    } else if (STREQ(s, "DECEMBER") || STREQ(s, "Dec") || STREQ(s, "dec")) {
	return DECEMBER;
    } else {
	return -1;
    }
}
